# Configuration file for pyB12MPS
# Bridge12 Technologies, Inc.

# TCP Server
HOST = 'localhost'
PORT = 50007

# Serial Port Settings
defaultSerialPort = 'COM5'
autoDetectSerialPort = True
initializeOnStart = True
serialDelay = 0.00 # in seconds

# System Ready String
systemReadyString = 'System Ready'
